#!/usr/bin/env python

"""
DOM macros for virtual libxml2mod node methods and properties.
"""

import xml.dom
import libxml2mod

class TemporaryNode:
    def __init__(self, ns, name):
        self._ns = ns
        self.name = name
        self.type = "attribute"
        self.nodeValue = None

    def ns(self):
        return self._ns

# NOTE: libxml2 seems to use UTF-8 throughout.

def from_unicode(s):
    if type(s) == type(u""):
        return s.encode("utf-8")
    else:
        return s

def to_unicode(s):
    if type(s) == type(""):
        return unicode(s, encoding="utf-8")
    else:
        return s

def _get_prefix_and_localName(name):
    t = name.split(":")
    if len(t) == 1:
        return None, name
    elif len(t) == 2:
        return t
    else:
        # NOTE: Should raise an exception.
        return None, None

_nodeTypes = {
    "attribute" : xml.dom.Node.ATTRIBUTE_NODE,
    "comment" : xml.dom.Node.COMMENT_NODE,
    "document_xml" : xml.dom.Node.DOCUMENT_NODE,
    "doctype" : xml.dom.Node.DOCUMENT_TYPE_NODE,
    "dtd" : xml.dom.Node.DOCUMENT_TYPE_NODE, # NOTE: Needs verifying.
    "element" : xml.dom.Node.ELEMENT_NODE,
    "entity" : xml.dom.Node.ENTITY_NODE,
    "entity_ref" : xml.dom.Node.ENTITY_REFERENCE_NODE,
    "notation" : xml.dom.Node.NOTATION_NODE,
    "pi" : xml.dom.Node.PROCESSING_INSTRUCTION_NODE,
    "text" : xml.dom.Node.TEXT_NODE
    }

def Node_ownerDocument(node):
    return libxml2mod.doc(node) or node

def Node_nodeType(node):
    global _nodesTypes
    return _nodeTypes[libxml2mod.type(node)]

def Node_childNodes(node):

    # NOTE: Consider a generator instead.

    child_nodes = []
    node = libxml2mod.children(node)
    while node is not None:
        child_nodes.append(node)
        node = libxml2mod.next(node)
    return child_nodes

def Node_attributes(node):
    attributes = {}
    node = libxml2mod.properties(node)
    while node is not None:
        ns = libxml2mod.xmlNodeGetNs(node)
        if ns is not None:
            attributes[(libxml2mod.xmlNodeGetContent(ns), libxml2mod.name(node))] = node
        else:
            attributes[(None, libxml2mod.name(node))] = node
        node = libxml2mod.next(node)
    return attributes

def Node_namespaceURI(node):
    ns = libxml2mod.xmlNodeGetNs(node)
    if ns is not None:
        return to_unicode(libxml2mod.xmlNodeGetContent(ns))
    else:
        return None

def Node_nodeValue(node):
    return to_unicode(libxml2mod.xmlNodeGetContent(node))

def Node_prefix(node):
    ns = libxml2mod.xmlNodeGetNs(node)
    if ns is not None:
        return to_unicode(libxml2mod.name(ns))
    else:
        return None

def Node_nodeName(node):
    prefix = Node_prefix(node)
    if prefix is not None:
        return prefix + ":" + Node_localName(node)
    else:
        return Node_localName(node)

def Node_tagName(node):
    if libxml2mod.type(node) == "element":
        return Node_nodeName(node)
    else:
        return None

def Node_localName(node):
    return to_unicode(libxml2mod.name(node))

def Node_parentNode(node):
    if libxml2mod.type(node) == "document_xml":
        return None
    else:
        return libxml2mod.parent(node)

def Node_previousSibling(node):
    if libxml2mod.prev(node) is not None:
        return libxml2mod.prev(node)
    else:
        return None

def Node_nextSibling(node):
    if libxml2mod.next(node) is not None:
        return libxml2mod.next(node)
    else:
        return None

def Node_hasAttributeNS(node, ns, localName):
    return Node_getAttributeNS(ns, localName) is not None

def Node_hasAttribute(node, name):
    return Node_getAttribute(name) is not None

def Node_getAttributeNS(node, ns, localName):
    return to_unicode(libxml2mod.xmlGetNsProp(node, localName, ns))

def Node_getAttribute(node, name):
    return to_unicode(libxml2mod.xmlGetProp(node, name))

def Node_getAttributeNodeNS(node, ns, localName):
    # NOTE: Needs verifying.
    return libxml2mod.xmlGetNsProp(node, localName, ns)

def Node_getAttributeNode(node, name):
    # NOTE: Needs verifying.
    return libxml2mod.xmlGetProp(node, name)

def Node_setAttributeNS(node, ns, name, value):
    # NOTE: Need to convert from Unicode.
    ns, name, value = map(from_unicode, [ns, name, value])

    prefix, localName = _get_prefix_and_localName(name)

    # NOTE: Might need to be xmlSetNsProp.
    if prefix is not None:
        libxml2mod.xmlNewNsProp(node, libxml2mod.xmlNewNs(node, ns, prefix), localName, value)
    elif ns is not None and ns == libxml2mod.xmlNodeGetContent(libxml2mod.xmlNodeGetNs(node)):
        libxml2mod.xmlNewNsProp(node, libxml2mod.xmlNodeGetNs(node), localName, value)
    else:
        # NOTE: Needs verifying: what should happen to the namespace?
        # NOTE: This also catches the case where None is the element's
        # NOTE: namespace and is also used for the attribute.
        libxml2mod.xmlNewNsProp(node, None, localName, value)

def Node_setAttribute(node, name, value):
    # NOTE: Need to convert from Unicode.
    name, value = map(from_unicode, [name, value])

    libxml2mod.xmlSetProp(node, name, value)

def _add_node(node, tmp):
    if tmp.ns is not None:
        child = libxml2mod.xmlNewNsProp(node, None, Node_localName(tmp), None)
        ns = libxml2mod.xmlNewNs(child, Node_namespaceURI(tmp), Node_prefix(tmp))
        libxml2mod.xmlNodeSetNs(child, ns)
    else:
        child = libxml2mod.xmlNewProp(node, libxml2mod.name(tmp))

    return child

def Node_setAttributeNodeNS(node, ns, name, attr):
    # NOTE: Not actually putting the node on the element.
    Node_setAttributeNS(node, ns, name, attr.nodeValue) # Node_nodeValue(attr)

def Node_setAttributeNode(node, name, attr):
    # NOTE: Not actually putting the node on the element.
    Node_setAttribute(node, name, attr.nodeValue) # Node_nodeValue(attr)

def Node_createElementNS(node, ns, name):
    # NOTE: Need to convert from Unicode.
    ns, name = map(from_unicode, [ns, name])

    prefix, localName = _get_prefix_and_localName(name)
    new_node = libxml2mod.xmlNewNode(localName)
    # NOTE: Does it make sense to set the namespace if it is empty?
    if ns is not None:
        new_ns = libxml2mod.xmlNewNs(new_node, ns, prefix)
        libxml2mod.xmlSetNs(new_node, new_ns)
    return new_node

def Node_createElement(node, name):
    # NOTE: Need to convert from Unicode.
    name = from_unicode(name)

    new_node = libxml2mod.xmlNewNode(name)
    return new_node

def Node_createAttributeNS(node, ns, name):
    # NOTE: Need to convert from Unicode.
    ns, name = map(from_unicode, [ns, name])

    prefix, localName = _get_prefix_and_localName(name)
    # NOTE: Does it make sense to set the namespace if it is empty?
    if ns is not None:
        new_ns = libxml2mod.xmlNewNs(new_node, ns, prefix)
    else:
        new_ns = None
    return TemporaryNode(new_ns, localName)

def Node_createAttribute(node, name):
    # NOTE: Need to convert from Unicode.
    name = from_unicode(name)

    return TemporaryNode(None, name)

def Node_createTextNode(node, value):
    # NOTE: Need to convert from Unicode.
    value = from_unicode(value)

    return libxml2mod.xmlNewText(value)

def Node_createComment(node, value):
    # NOTE: Need to convert from Unicode.
    value = from_unicode(value)

    return libxml2mod.xmlNewComment(value)

def Node_insertBefore(node, tmp, oldNode):
    if not isinstance(tmp, TemporaryNode):
        return libxml2mod.xmlAddPrevSibling(oldNode, tmp)
    else:
        return None

def Node_replaceChild(node, tmp, oldNode):
    if not isinstance(tmp, TemporaryNode):
        return libxml2mod.xmlReplaceNode(oldNode, tmp)
    else:
        return None

def Node_appendChild(node, tmp):
    return libxml2mod.xmlAddChild(node, tmp)

def Node_removeChild(node, child):
    libxml2mod.unlinkNode(child)

def Node_xpath(node, expr, variables=None, namespaces=None):
    context = libxml2mod.xmlXPathNewContext(Node_ownerDocument(node))
    # NOTE: Discover namespaces from the node.
    for prefix, ns in (namespaces or {}).items():
        libxml2mod.xmlXPathRegisterNs(context, prefix, ns)
    # NOTE: May need to tidy up the context.
    return libxml2mod.xmlXPathEval(expr, context)

# Utility functions.

def createDocumentType(localName, publicId, systemId):
    return None

def createDocument(namespaceURI, localName, doctype):
    # NOTE: Fixed to use version 1.0 only.
    d = libxml2mod.xmlNewDoc("1.0")
    if localName is not None:
        root = Node_createElementNS(d, namespaceURI, localName)
        Node_appendChild(d, root)
    return d

def parse(stream_or_string):
    if hasattr(stream_or_string, "read"):
        stream = stream_or_string
        return parseString(stream.read())
    else:
        return parseFile(stream_or_string)

def parseFile(s):
    # NOTE: Switching off validation and remote DTD resolution.
    context = libxml2mod.xmlCreateFileParserCtxt(s)
    libxml2mod.xmlParserSetValidate(context, 0)
    libxml2mod.xmlCtxtUseOptions(context, 0)
    libxml2mod.xmlParseDocument(context)
    return libxml2mod.xmlParserGetDoc(context)

def parseString(s):
    # NOTE: Switching off validation and remote DTD resolution.
    context = libxml2mod.xmlCreateMemoryParserCtxt(s, len(s))
    libxml2mod.xmlParserSetValidate(context, 0)
    libxml2mod.xmlCtxtUseOptions(context, 0)
    libxml2mod.xmlParseDocument(context)
    return libxml2mod.xmlParserGetDoc(context)

def parseURI(uri):
    context = libxml2mod.xmlCreateURLParserCtxt(url)
    libxml2mod.xmlParserSetValidate(context, 0)
    libxml2mod.xmlCtxtUseOptions(context, 0)
    libxml2mod.xmlParseDocument(context)
    return libxml2mod.xmlParserGetDoc(context)

def toFile(doc, s):
    libxml2mod.xmlSaveFile(s, doc)

# vim: tabstop=4 expandtab shiftwidth=4
